@@ -1,7 +1,7 @@
-/* $Id: getifstats.c,v 1.13 2015/06/09 15:33:24 nanard Exp $ */
+/* $Id: getifstats.c,v 1.14 2018/03/13 23:05:21 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2013 Thomas Bernard
+ * (c) 2006-2018 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -48,7 +48,7 @@ getifstats(const char * ifname, struct ifdata * data)
 	if(!ifname || ifname[0]=='\0')
 		return -1;
 #ifdef ENABLE_GETIFSTATS_CACHING
-	current_time = time(NULL);
+	current_time = upnp_time();
 	if(current_time == ((time_t)-1)) {
 		syslog(LOG_ERR, "getifstats() : time() error : %m");
 	} else {
