@@ -1,8 +1,8 @@
-/* $Id: minissdp.c,v 1.89 2017/12/12 11:41:02 nanard Exp $ */
+/* $Id: minissdp.c,v 1.93 2018/04/22 19:36:58 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
- * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2017 Thomas Bernard
+ * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
+ * (c) 2006-2018 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -190,7 +190,7 @@ OpenAndConfSSDPReceiveSocket(int ipv6)
 		struct sockaddr_in * saddr = (struct sockaddr_in *)&sockname;
 		saddr->sin_family = AF_INET;
 		saddr->sin_port = htons(SSDP_PORT);
-		/* NOTE : it seems it doesnt work when binding on the specific address */
+		/* NOTE : it seems it doesn't work when binding on the specific address */
 		/*saddr->sin_addr.s_addr = inet_addr(UPNP_MCAST_ADDR);*/
 		saddr->sin_addr.s_addr = htonl(INADDR_ANY);
 		/*saddr->sin_addr.s_addr = inet_addr(ifaddr);*/
@@ -237,7 +237,7 @@ OpenAndConfSSDPReceiveSocket(int ipv6)
 #if defined(SO_BINDTODEVICE) && !defined(MULTIPLE_EXTERNAL_IP)
 	/* One and only one LAN interface */
 	if(lan_addrs.lh_first != NULL && lan_addrs.lh_first->list.le_next == NULL
-	   && strlen(lan_addrs.lh_first->ifname) > 0)
+	   && lan_addrs.lh_first->ifname[0] != '\0')
 	{
 		if(setsockopt(s, SOL_SOCKET, SO_BINDTODEVICE,
 		              lan_addrs.lh_first->ifname,
@@ -265,7 +265,7 @@ OpenAndConfSSDPReceiveSocket(int ipv6)
 			{
 				syslog(LOG_WARNING,
 				       "Failed to add IPv6 multicast membership for interface %s",
-				       lan_addr->str ? lan_addr->str : "NULL");
+				       strlen(lan_addr->str) ? lan_addr->str : "NULL");
 			}
 		}
 	}
@@ -1002,7 +1002,7 @@ ProcessSSDPData(int s, const char *bufr, int n,
 	{
 		if(lan_addr != NULL)
 		{
-			if(lan_addr->index != (unsigned)source_if)
+			if(lan_addr->index != (unsigned)source_if && lan_addr->index != 0)
 			{
 				syslog(LOG_WARNING, "interface index not matching %u != %d", lan_addr->index, source_if);
 			}
@@ -1496,20 +1496,23 @@ SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
 	}
 	for(i = 0; known_service_types[i].s; i++) {
 		buffer[0] = 4;	/* request type 4 : submit service */
-		/* 4 strings following : ST (service type), USN, Server, Location */
-		p = buffer + 1;
-		l = (int)strlen(known_service_types[i].s);
-		if(i > 0)
-			l++;
-		CODELENGTH(l, p);
-		memcpy(p, known_service_types[i].s, l);
-		if(i > 0)
-			p[l-1] = '1';
-		p += l;
 		if(i==0)
 			ver_str[0] = '\0';
 		else
 			snprintf(ver_str, sizeof(ver_str), "%d", known_service_types[i].version);
+		/* 4 strings following : ST (service type), USN, Server, Location */
+		p = buffer + 1;
+		l = snprintf(strbuf, sizeof(strbuf), "%s%s",
+		             known_service_types[i].s, ver_str);
+		if(l<0) {
+			syslog(LOG_WARNING, "SubmitServicesToMiniSSDPD: snprintf %m");
+			continue;
+		} else if((unsigned)l>=sizeof(strbuf)) {
+			l = sizeof(strbuf) - 1;
+		}
+		CODELENGTH(l, p);
+		memcpy(p, strbuf, l);
+		p += l;
 		l = snprintf(strbuf, sizeof(strbuf), "%s::%s%s",
 		             known_service_types[i].uuid, known_service_types[i].s, ver_str);
 		if(l<0) {
@@ -1557,6 +1560,7 @@ SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
 		}
 	}
  	close(s);
+	syslog(LOG_DEBUG, "%d service submitted to MiniSSDPd", i);
 	return 0;
 }
 
