@@ -1,12 +1,12 @@
-/* $Id: miniupnpdpath.h,v 1.8 2011/05/20 17:51:23 nanard Exp $ */
+/* $Id: miniupnpdpath.h,v 1.9 2012/09/27 15:47:15 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2011 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __MINIUPNPDPATH_H__
-#define __MINIUPNPDPATH_H__
+#ifndef MINIUPNPDPATH_H_INCLUDED
+#define MINIUPNPDPATH_H_INCLUDED
 
 #include "config.h"
 
