@@ -1,7 +1,7 @@
-/* $Id: iptcrdr.c,v 1.54 2015/03/09 10:02:54 nanard Exp $ */
+/* $Id: iptcrdr.c,v 1.59 2016/03/08 09:23:52 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2015 Thomas Bernard
+ * (c) 2006-2016 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 #include <stdio.h>
@@ -72,6 +72,14 @@ static int
 add_filter_rule(int proto, const char * rhost,
                 const char * iaddr, unsigned short iport);
 
+#ifdef ENABLE_PORT_TRIGGERING
+static int
+addmasqueraderule(int proto,
+           unsigned short eport,
+           const char * iaddr, unsigned short iport,
+           const char * rhost/*, const char * extif*/);
+#endif /* ENABLE_PORT_TRIGGERING */
+
 static int
 addpeernatrule(int proto,
            const char * eaddr, unsigned short eport,
@@ -83,9 +91,24 @@ addpeerdscprule(int proto, unsigned char dscp,
            const char * iaddr, unsigned short iport,
            const char * rhost, unsigned short rport);
 
-/* dummy init and shutdown functions */
+/* dummy init and shutdown functions
+ * Only test iptc_init() */
 int init_redirect(void)
 {
+	IPTC_HANDLE h;
+
+	h = iptc_init("nat");
+	if(!h) {
+		syslog(LOG_ERR, "iptc_init() failed : %s",
+		       iptc_strerror(errno));
+		return -1;
+	} else {
+#ifdef IPTABLES_143
+		iptc_free(h);
+#else
+		iptc_free(&h);
+#endif
+	}
 	return 0;
 }
 
@@ -186,31 +209,6 @@ get_redirect_desc(unsigned short eport, int proto,
 		*timestamp = 0;
 }
 
-#if USE_INDEX_FROM_DESC_LIST
-static int
-get_redirect_desc_by_index(int index, unsigned short * eport, int * proto,
-                  char * desc, int desclen, unsigned int * timestamp)
-{
-	int i = 0;
-	struct rdr_desc * p;
-	if(!desc || (desclen == 0))
-		return -1;
-	for(p = rdr_desc_list; p; p = p->next, i++)
-	{
-		if(i == index)
-		{
-			*eport = p->eport;
-			*proto = (int)p->proto;
-			strncpy(desc, p->str, desclen);
-			if(timestamp)
-				*timestamp = p->timestamp;
-			return 0;
-		}
-	}
-	return -1;
-}
-#endif
-
 /* add_redirect_rule2() */
 int
 add_redirect_rule2(const char * ifname,
@@ -222,12 +220,27 @@ add_redirect_rule2(const char * ifname,
 	UNUSED(ifname);
 
 	r = addnatrule(proto, eport, iaddr, iport, rhost);
-	if(r >= 0)
+	if(r >= 0) {
 		add_redirect_desc(eport, proto, desc, timestamp);
+#ifdef ENABLE_PORT_TRIGGERING
+		/* http://www.netfilter.org/documentation/HOWTO/NAT-HOWTO-6.html#ss6.3
+		 * The default behavior is to alter the connection as little
+		 * as possible, within the constraints of the rule given by
+		 * the user.
+		 * This means we won't remap ports unless we have to. */
+		if(iport != eport) {
+			/* TODO : check if this should be done only with UDP */
+			r = addmasqueraderule(proto, eport, iaddr, iport, rhost/*, ifname*/);
+			if(r < 0) {
+				syslog(LOG_NOTICE, "add_redirect_rule2(): addmasqueraderule returned %d", r);
+			}
+		}
+#endif /* ENABLE_PORT_TRIGGERING */
+	}
 	return r;
 }
 
-/* add_redirect_rule2() */
+/* add_peer_redirect_rule2() */
 int
 add_peer_redirect_rule2(const char * ifname,
                    const char * rhost, unsigned short rport,
@@ -312,9 +325,8 @@ get_nat_redirect_rule(const char * nat_chain_name, const char * ifname, unsigned
 	h = iptc_init("nat");
 	if(!h)
 	{
-		syslog(LOG_ERR, "get_redirect_rule() : "
-		                "iptc_init() failed : %s",
-		       iptc_strerror(errno));
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "get_nat_redirect_rule", iptc_strerror(errno));
 		return -1;
 	}
 	if(!iptc_is_chain(nat_chain_name, h))
@@ -390,15 +402,6 @@ get_redirect_rule_by_index(int index,
                            u_int64_t * packets, u_int64_t * bytes)
 {
 	int r = -1;
-#if USE_INDEX_FROM_DESC_LIST
-	r = get_redirect_desc_by_index(index, eport, proto,
-	                               desc, desclen, timestamp);
-	if (r==0)
-	{
-		r = get_redirect_rule(ifname, *eport, *proto, iaddr, iaddrlen, iport,
-				      0, 0, packets, bytes);
-	}
-#else
 	int i = 0;
 	IPTC_HANDLE h;
 	const struct ipt_entry * e;
@@ -410,9 +413,8 @@ get_redirect_rule_by_index(int index,
 	h = iptc_init("nat");
 	if(!h)
 	{
-		syslog(LOG_ERR, "get_redirect_rule_by_index() : "
-		                "iptc_init() failed : %s",
-		       iptc_strerror(errno));
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "get_redirect_rule_by_index", iptc_strerror(errno));
 		return -1;
 	}
 	if(!iptc_is_chain(miniupnpd_nat_chain, h))
@@ -475,7 +477,6 @@ get_redirect_rule_by_index(int index,
 		iptc_free(h);
 #else
 		iptc_free(&h);
-#endif
 #endif
 	return r;
 }
@@ -492,15 +493,6 @@ get_peer_rule_by_index(int index,
                            u_int64_t * packets, u_int64_t * bytes)
 {
 	int r = -1;
-#if USE_INDEX_FROM_DESC_LIST && 0
-	r = get_redirect_desc_by_index(index, eport, proto,
-	                               desc, desclen, timestamp);
-	if (r==0)
-	{
-		r = get_redirect_rule(ifname, *eport, *proto, iaddr, iaddrlen, iport,
-				      0, 0, packets, bytes);
-	}
-#else
 	int i = 0;
 	IPTC_HANDLE h;
 	const struct ipt_entry * e;
@@ -512,23 +504,22 @@ get_peer_rule_by_index(int index,
 	h = iptc_init("nat");
 	if(!h)
 	{
-		syslog(LOG_ERR, "get_peer_rule_by_index() : "
-		                "iptc_init() failed : %s",
-		       iptc_strerror(errno));
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "get_peer_rule_by_index", iptc_strerror(errno));
 		return -1;
 	}
-	if(!iptc_is_chain(miniupnpd_peer_chain, h))
+	if(!iptc_is_chain(miniupnpd_nat_postrouting_chain, h))
 	{
-		syslog(LOG_ERR, "chain %s not found", miniupnpd_peer_chain);
+		syslog(LOG_ERR, "chain %s not found", miniupnpd_nat_postrouting_chain);
 	}
 	else
 	{
 #ifdef IPTABLES_143
-		for(e = iptc_first_rule(miniupnpd_peer_chain, h);
+		for(e = iptc_first_rule(miniupnpd_nat_postrouting_chain, h);
 		    e;
 			e = iptc_next_rule(e, h))
 #else
-		for(e = iptc_first_rule(miniupnpd_peer_chain, &h);
+		for(e = iptc_first_rule(miniupnpd_nat_postrouting_chain, &h);
 		    e;
 			e = iptc_next_rule(e, &h))
 #endif
@@ -589,7 +580,6 @@ get_peer_rule_by_index(int index,
 		iptc_free(h);
 #else
 		iptc_free(&h);
-#endif
 #endif
 	return r;
 }
@@ -650,9 +640,8 @@ delete_redirect_and_filter_rules(unsigned short eport, int proto)
 	h = iptc_init("nat");
 	if(!h)
 	{
-		syslog(LOG_ERR, "delete_redirect_and_filter_rules() : "
-		                "iptc_init() failed : %s",
-		       iptc_strerror(errno));
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "delete_redirect_and_filter_rules", iptc_strerror(errno));
 		return -1;
 	}
 	/* First step : find the right nat rule */
@@ -774,11 +763,11 @@ delete_redirect_and_filter_rules(unsigned short eport, int proto)
 		i = 0;
 		/* we must find the right index for the filter rule */
 #ifdef IPTABLES_143
-		for(e = iptc_first_rule(miniupnpd_peer_chain, h);
+		for(e = iptc_first_rule(miniupnpd_nat_postrouting_chain, h);
 		    e;
 			e = iptc_next_rule(e, h), i++)
 #else
-		for(e = iptc_first_rule(miniupnpd_peer_chain, &h);
+		for(e = iptc_first_rule(miniupnpd_nat_postrouting_chain, &h);
 		    e;
 			e = iptc_next_rule(e, &h), i++)
 #endif
@@ -787,6 +776,9 @@ delete_redirect_and_filter_rules(unsigned short eport, int proto)
 			{
 				target = (void *)e + e->target_offset;
 				mr = (const struct ip_nat_multi_range *)&target->data[0];
+				syslog(LOG_DEBUG, "postrouting rule #%u: %s %s %hu",
+				       i, target->u.user.name, inet_ntoa(e->ip.src), ntohs(mr->range[0].min.all));
+				/* target->u.user.name SNAT / MASQUERADE */
 				if (eport != ntohs(mr->range[0].min.all)) {
 					continue;
 				}
@@ -807,7 +799,7 @@ delete_redirect_and_filter_rules(unsigned short eport, int proto)
 
 				index = i;
 				syslog(LOG_INFO, "Trying to delete peer rule at index %u", index);
-				r2 = delete_rule_and_commit(index, h, miniupnpd_peer_chain, "delete_peer_rule");
+				r2 = delete_rule_and_commit(index, h, miniupnpd_nat_postrouting_chain, "delete_peer_rule");
 				h = NULL;
 				break;
 			}
@@ -1003,6 +995,31 @@ get_dscp_target(unsigned char dscp)
 	return target;
 }
 
+#ifdef ENABLE_PORT_TRIGGERING
+static struct ipt_entry_target *
+get_masquerade_target(unsigned short port)
+{
+	struct ipt_entry_target * target;
+	struct ip_nat_multi_range * mr;
+	struct ip_nat_range * range;
+	size_t size;
+
+	size =   IPT_ALIGN(sizeof(struct ipt_entry_target))
+	       + IPT_ALIGN(sizeof(struct ip_nat_multi_range));
+	target = calloc(1, size);
+	target->u.target_size = size;
+	strncpy(target->u.user.name, "MASQUERADE", sizeof(target->u.user.name));
+	/* one ip_nat_range already included in ip_nat_multi_range */
+	mr = (struct ip_nat_multi_range *)&target->data[0];
+	mr->rangesize = 1;
+	range = &mr->range[0];
+	range->min.tcp.port = range->max.tcp.port = htons(port);
+	/*range->min.all = range->max.all = htons(port);*/
+	range->flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
+	return target;
+}
+#endif /* ENABLE_PORT_TRIGGERING */
+
 /* iptc_init_verify_and_append()
  * return 0 on success, -1 on failure */
 static int
@@ -1015,13 +1032,13 @@ iptc_init_verify_and_append(const char * table,
 	h = iptc_init(table);
 	if(!h)
 	{
-		syslog(LOG_ERR, "%s : iptc_init() error : %s\n",
+		syslog(LOG_ERR, "%s() : iptc_init() error : %s\n",
 		       logcaller, iptc_strerror(errno));
 		return -1;
 	}
 	if(!iptc_is_chain(miniupnpd_chain, h))
 	{
-		syslog(LOG_ERR, "%s : chain %s not found",
+		syslog(LOG_ERR, "%s() : chain %s not found",
 		       logcaller, miniupnpd_chain);
 		if(h)
 #ifdef IPTABLES_143
@@ -1038,7 +1055,7 @@ iptc_init_verify_and_append(const char * table,
 	if(!iptc_append_entry(miniupnpd_chain, e, &h))
 #endif
 	{
-		syslog(LOG_ERR, "%s : iptc_append_entry() error : %s\n",
+		syslog(LOG_ERR, "%s() : iptc_append_entry() error : %s\n",
 		       logcaller, iptc_strerror(errno));
 		if(h)
 #ifdef IPTABLES_143
@@ -1054,7 +1071,7 @@ iptc_init_verify_and_append(const char * table,
 	if(!iptc_commit(&h))
 #endif
 	{
-		syslog(LOG_ERR, "%s : iptc_commit() error : %s\n",
+		syslog(LOG_ERR, "%s() : iptc_commit() error : %s\n",
 		       logcaller, iptc_strerror(errno));
 		if(h)
 #ifdef IPTABLES_143
@@ -1074,7 +1091,7 @@ iptc_init_verify_and_append(const char * table,
 }
 
 /* add nat rule
- * iptables -t nat -A MINIUPNPD -p proto --dport eport -j DNAT --to iaddr:iport
+ * iptables -t nat -A MINIUPNPD -p <proto> [-s <rhost>] --dport <eport> -j DNAT --to <iaddr>:<iport>
  * */
 static int
 addnatrule(int proto, unsigned short eport,
@@ -1127,16 +1144,103 @@ addnatrule(int proto, unsigned short eport,
 		e->ip.smsk.s_addr = INADDR_NONE;
 	}
 
-	r = iptc_init_verify_and_append("nat", miniupnpd_nat_chain, e, "addnatrule()");
+	r = iptc_init_verify_and_append("nat", miniupnpd_nat_chain, e, "addnatrule");
 	free(target);
 	free(match);
 	free(e);
 	return r;
 }
 
-/* iptables -t nat -A MINIUPNPD-PCP-PEER -s iaddr -d rhost
- *    -p proto --sport iport --dport rport -j SNAT
- *    --to-source ext_ip:eport */
+/* for "Port Triggering"
+ * Section 2.5.16 figure 2.2 in UPnP-gw-WANIPConnection-v2-Service.pdf
+ *
+ *   When a control point creates a port forwarding rule with AddPortMapping()
+ *   action for inbound traffic , this rule MUST also be applied when NAT port
+ *   triggering occurs for outbound traffic.
+ *
+ * iptables -t nat -A MINIUPNPD-POSTROUTING {-o <extif>} -s <iaddr>
+ *          -p <proto> [-d <rhost>] --sport <iport> -j MASQUERADE --to-ports <eport>
+ */
+#ifdef ENABLE_PORT_TRIGGERING
+static int
+addmasqueraderule(int proto,
+           unsigned short eport,
+           const char * iaddr, unsigned short iport,
+           const char * rhost/*, const char * extif*/)
+{
+	int r = 0;
+	struct ipt_entry * e;
+	struct ipt_entry * tmp;
+	struct ipt_entry_match *match = NULL;
+	struct ipt_entry_target *target = NULL;
+
+	e = calloc(1, sizeof(struct ipt_entry));
+	if(!e) {
+		syslog(LOG_ERR, "%s: calloc(%d) error", "addmasqueraderule",
+		       (int)sizeof(struct ipt_entry));
+		return -1;
+	}
+	e->ip.proto = proto;
+	if(proto == IPPROTO_TCP) {
+		match = get_tcp_match(0, iport);
+	} else {
+		match = get_udp_match(0, iport);
+	}
+	e->nfcache = NFC_IP_DST_PT;
+	target = get_masquerade_target(eport);
+	e->nfcache |= NFC_UNKNOWN;
+	tmp = realloc(e, sizeof(struct ipt_entry)
+	               + match->u.match_size
+				   + target->u.target_size);
+	if(!tmp) {
+		syslog(LOG_ERR, "%s: realloc(%d) error", "addmasqueraderule",
+		       (int)(sizeof(struct ipt_entry) + match->u.match_size + target->u.target_size));
+		free(e);
+		free(match);
+		free(target);
+		return -1;
+	}
+	e = tmp;
+	memcpy(e->elems, match, match->u.match_size);
+	memcpy(e->elems + match->u.match_size, target, target->u.target_size);
+	e->target_offset = sizeof(struct ipt_entry)
+	                   + match->u.match_size;
+	e->next_offset = sizeof(struct ipt_entry)
+	                 + match->u.match_size
+					 + target->u.target_size;
+#if 0
+	/* do not add outiface (-o) to rule, as the MINIUPNPD-POSTROUTING chain
+	 * should already have matched it */
+	if(extif != NULL) {
+		strncpy(e->ip.outiface, extif, sizeof(e->ip.outiface));
+		memset(e->ip.outiface_mask, 0xff, strlen(e->ip.outiface) + 1);/* Include nul-terminator in match */
+	}
+#endif
+	/* internal host */
+	if(iaddr && (iaddr[0] != '\0') && (0 != strcmp(iaddr, "*")))
+	{
+		e->ip.src.s_addr = inet_addr(iaddr);
+		e->ip.smsk.s_addr = INADDR_NONE;
+	}
+	/* remote host */
+	if(rhost && (rhost[0] != '\0') && (0 != strcmp(rhost, "*"))) {
+		e->ip.dst.s_addr = inet_addr(rhost);
+		e->ip.dmsk.s_addr = INADDR_NONE;
+	}
+
+	r = iptc_init_verify_and_append("nat", miniupnpd_nat_postrouting_chain, e, "addmasqueraderule");
+	free(target);
+	free(match);
+	free(e);
+	return r;
+}
+#endif /* ENABLE_PORT_TRIGGERING */
+
+/* called by add_peer_redirect_rule2()
+ *
+ * iptables -t nat -A MINIUPNPD-POSTROUTING -s <iaddr> -d <rhost>
+ *    -p <proto> --sport <iport> --dport <rport> -j SNAT
+ *    --to-source <eaddr>:<eport> */
 static int
 addpeernatrule(int proto,
            const char * eaddr, unsigned short eport,
@@ -1198,15 +1302,16 @@ addpeernatrule(int proto,
 		e->ip.dmsk.s_addr = INADDR_NONE;
 	}
 
-	r = iptc_init_verify_and_append("nat", miniupnpd_peer_chain, e, "addpeernatrule()");
+	r = iptc_init_verify_and_append("nat", miniupnpd_nat_postrouting_chain, e, "addpeernatrule");
 	free(target);
 	free(match);
 	free(e);
 	return r;
 }
 
-/* iptables -t mangle -A MINIUPNPD -s iaddr -d rhost
- *    -p proto --sport iport --dport rport -j DSCP
+/* called by add_peer_dscp_rule2()
+ * iptables -t mangle -A MINIUPNPD -s <iaddr> -d <rhost>
+ *    -p <proto> --sport <iport> --dport <rport> -j DSCP
  *    --set-dscp 0xXXXX                   */
 static int
 addpeerdscprule(int proto, unsigned char dscp,
@@ -1269,7 +1374,7 @@ addpeerdscprule(int proto, unsigned char dscp,
 	}
 
 	r = iptc_init_verify_and_append("mangle", miniupnpd_nat_chain, e,
-	                                "addpeerDSCPrule()");
+	                                "addpeerDSCPrule");
 	free(target);
 	free(match);
 	free(e);
@@ -1292,7 +1397,7 @@ get_accept_target(void)
 }
 
 /* add_filter_rule()
- * */
+ * iptables -t filter -A MINIUPNPD [-s <rhost>] -p <proto> -d <iaddr> --dport <iport> -j ACCEPT */
 static int
 add_filter_rule(int proto, const char * rhost,
                 const char * iaddr, unsigned short iport)
@@ -1346,7 +1451,7 @@ add_filter_rule(int proto, const char * rhost,
 		e->ip.smsk.s_addr = INADDR_NONE;
 	}
 
-	r = iptc_init_verify_and_append("filter", miniupnpd_forward_chain, e, "add_filter_rule()");
+	r = iptc_init_verify_and_append("filter", miniupnpd_forward_chain, e, "add_filter_rule");
 	free(target);
 	free(match);
 	free(e);
@@ -1371,16 +1476,15 @@ get_portmappings_in_range(unsigned short startport, unsigned short endport,
 	array = calloc(capacity, sizeof(unsigned short));
 	if(!array)
 	{
-		syslog(LOG_ERR, "get_portmappings_in_range() : calloc error");
+		syslog(LOG_ERR, "%s() : calloc error", "get_portmappings_in_range");
 		return NULL;
 	}
 
 	h = iptc_init("nat");
 	if(!h)
 	{
-		syslog(LOG_ERR, "get_redirect_rule_by_index() : "
-		                "iptc_init() failed : %s",
-		       iptc_strerror(errno));
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "get_portmappings_in_range", iptc_strerror(errno));
 		free(array);
 		return NULL;
 	}
@@ -1450,17 +1554,260 @@ get_portmappings_in_range(unsigned short startport, unsigned short endport,
 	return array;
 }
 
+int
+update_portmapping_desc_timestamp(const char * ifname,
+                   unsigned short eport, int proto,
+                   const char * desc, unsigned int timestamp)
+{
+	UNUSED(ifname);
+	del_redirect_desc(eport, proto);
+	add_redirect_desc(eport, proto, desc, timestamp);
+	return 0;
+}
+
+static int
+update_rule_and_commit(const char * table, const char * chain,
+                       unsigned index, const struct ipt_entry * e)
+{
+	IPTC_HANDLE h;
+	int r = 0;
+
+	h = iptc_init(table);
+	if(!h)
+	{
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "update_rule_and_commit", iptc_strerror(errno));
+		return -1;
+	}
+#ifdef IPTABLES_143
+	if(!iptc_replace_entry(chain, e, index, h))
+#else
+	if(!iptc_replace_entry(chain, e, index, &h))
+#endif
+	{
+		syslog(LOG_ERR, "%s(): iptc_replace_entry: %s",
+		       "update_rule_and_commit", iptc_strerror(errno));
+		r = -1;
+	}
+#ifdef IPTABLES_143
+	else if(!iptc_commit(h))
+#else
+	else if(!iptc_commit(&h))
+#endif
+	{
+		syslog(LOG_ERR, "%s(): iptc_commit: %s",
+		       "update_rule_and_commit", iptc_strerror(errno));
+		r = -1;
+	}
+#ifdef IPTABLES_143
+	iptc_free(h);
+#else
+	iptc_free(&h);
+#endif
+	return r;
+}
+
+int
+update_portmapping(const char * ifname, unsigned short eport, int proto,
+                   unsigned short iport, const char * desc,
+                   unsigned int timestamp)
+{
+	int r = 0;
+	int found = 0;
+	unsigned index = 0;
+	unsigned i = 0;
+	IPTC_HANDLE h;
+	const struct ipt_entry * e;
+	struct ipt_entry * new_e = NULL;
+	size_t entry_len;
+	const struct ipt_entry_target * target;
+	struct ip_nat_multi_range * mr;
+	const struct ipt_entry_match *match;
+	uint32_t iaddr = 0;
+	unsigned short old_iport = 0;
+
+	h = iptc_init("nat");
+	if(!h)
+	{
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "update_portmapping", iptc_strerror(errno));
+		return -1;
+	}
+	/* First step : find the right nat rule */
+	if(!iptc_is_chain(miniupnpd_nat_chain, h))
+	{
+		syslog(LOG_ERR, "chain %s not found", miniupnpd_nat_chain);
+		r = -1;
+	}
+	else
+	{
+#ifdef IPTABLES_143
+		for(e = iptc_first_rule(miniupnpd_nat_chain, h);
+		    e;
+			e = iptc_next_rule(e, h), i++)
+#else
+		for(e = iptc_first_rule(miniupnpd_nat_chain, &h);
+		    e;
+			e = iptc_next_rule(e, &h), i++)
+#endif
+		{
+			if(proto==e->ip.proto)
+			{
+				match = (const struct ipt_entry_match *)&e->elems;
+				if(0 == strncmp(match->u.user.name, "tcp", IPT_FUNCTION_MAXNAMELEN))
+				{
+					const struct ipt_tcp * info;
+					info = (const struct ipt_tcp *)match->data;
+					if(eport != info->dpts[0])
+						continue;
+				}
+				else
+				{
+					const struct ipt_udp * info;
+					info = (const struct ipt_udp *)match->data;
+					if(eport != info->dpts[0])
+						continue;
+				}
+				/* we found the right rule */
+				found = 1;
+				index = i;
+				target = (void *)e + e->target_offset;
+				mr = (struct ip_nat_multi_range *)&target->data[0];
+				iaddr = mr->range[0].min_ip;
+				old_iport = ntohs(mr->range[0].min.all);
+				entry_len = sizeof(struct ipt_entry) + match->u.match_size + target->u.target_size;
+				new_e = malloc(entry_len);
+				if(new_e == NULL) {
+					syslog(LOG_ERR, "%s: malloc(%u) error",
+					       "update_portmapping", (unsigned)entry_len);
+					r = -1;
+				}
+				else
+				{
+					memcpy(new_e, e, entry_len);
+				}
+				break;
+			}
+		}
+	}
+#ifdef IPTABLES_143
+	iptc_free(h);
+#else
+	iptc_free(&h);
+#endif
+	if(!found || r < 0)
+		return -1;
+	syslog(LOG_INFO, "Trying to update nat rule at index %u", index);
+	target = (void *)new_e + new_e->target_offset;
+	mr = (struct ip_nat_multi_range *)&target->data[0];
+	mr->range[0].min.all = mr->range[0].max.all = htons(iport);
+	/* first update the nat rule */
+	r = update_rule_and_commit("nat", miniupnpd_nat_chain, index, new_e);
+	free(new_e); new_e = NULL;
+	if(r < 0)
+		return r;
+
+	/* update filter rule */
+	h = iptc_init("filter");
+	if(!h)
+	{
+		syslog(LOG_ERR, "%s() : iptc_init() failed : %s",
+		       "update_portmapping", iptc_strerror(errno));
+		return -1;
+	}
+	i = 0; found = 0;
+	if(!iptc_is_chain(miniupnpd_forward_chain, h))
+	{
+		syslog(LOG_ERR, "chain %s not found", miniupnpd_forward_chain);
+	}
+	else
+	{
+		/* we must find the right index for the filter rule */
+#ifdef IPTABLES_143
+		for(e = iptc_first_rule(miniupnpd_forward_chain, h);
+		    e;
+			e = iptc_next_rule(e, h), i++)
+#else
+		for(e = iptc_first_rule(miniupnpd_forward_chain, &h);
+		    e;
+			e = iptc_next_rule(e, &h), i++)
+#endif
+		{
+			if(proto!=e->ip.proto)
+				continue;
+			target = (void *)e + e->target_offset;
+			match = (const struct ipt_entry_match *)&e->elems;
+			if(0 == strncmp(match->u.user.name, "tcp", IPT_FUNCTION_MAXNAMELEN))
+			{
+				const struct ipt_tcp * info;
+				info = (const struct ipt_tcp *)match->data;
+				if(old_iport != info->dpts[0])
+					continue;
+			}
+			else
+			{
+				const struct ipt_udp * info;
+				info = (const struct ipt_udp *)match->data;
+				if(old_iport != info->dpts[0])
+					continue;
+			}
+			if(iaddr != e->ip.dst.s_addr)
+				continue;
+			index = i;
+			found = 1;
+			entry_len = sizeof(struct ipt_entry) + match->u.match_size + target->u.target_size;
+			new_e = malloc(entry_len);
+			if(new_e == NULL) {
+				syslog(LOG_ERR, "%s: malloc(%u) error",
+				       "update_portmapping", (unsigned)entry_len);
+				r = -1;
+			} else {
+				memcpy(new_e, e, entry_len);
+			}
+			break;
+		}
+	}
+#ifdef IPTABLES_143
+	iptc_free(h);
+#else
+	iptc_free(&h);
+#endif
+	if(!found || r < 0)
+		return -1;
+
+	syslog(LOG_INFO, "Trying to update filter rule at index %u", index);
+	match = (struct ipt_entry_match *)&new_e->elems;
+	if(0 == strncmp(match->u.user.name, "tcp", IPT_FUNCTION_MAXNAMELEN))
+	{
+		struct ipt_tcp * info;
+		info = (struct ipt_tcp *)match->data;
+		info->dpts[0] = info->dpts[1] = iport;
+	}
+	else
+	{
+		struct ipt_udp * info;
+		info = (struct ipt_udp *)match->data;
+		info->dpts[0] = info->dpts[1] = iport;
+	}
+	r = update_rule_and_commit("filter", miniupnpd_forward_chain, index, new_e);
+	free(new_e); new_e = NULL;
+	if(r < 0)
+		return r;
+
+	return update_portmapping_desc_timestamp(ifname, eport, proto, desc, timestamp);
+}
+
 /* ================================ */
 #ifdef DEBUG
 static int
 print_match(const struct ipt_entry_match *match)
 {
-	printf("match %s\n", match->u.user.name);
+	printf("match %s :\n", match->u.user.name);
 	if(0 == strncmp(match->u.user.name, "tcp", IPT_FUNCTION_MAXNAMELEN))
 	{
 		struct ipt_tcp * tcpinfo;
 		tcpinfo = (struct ipt_tcp *)match->data;
-		printf("srcport = %hu:%hu dstport = %hu:%hu\n",
+		printf("  srcport = %hu:%hu dstport = %hu:%hu\n",
 		       tcpinfo->spts[0], tcpinfo->spts[1],
 			   tcpinfo->dpts[0], tcpinfo->dpts[1]);
 	}
@@ -1468,7 +1815,7 @@ print_match(const struct ipt_entry_match *match)
 	{
 		struct ipt_udp * udpinfo;
 		udpinfo = (struct ipt_udp *)match->data;
-		printf("srcport = %hu:%hu dstport = %hu:%hu\n",
+		printf("  srcport = %hu:%hu dstport = %hu:%hu\n",
 		       udpinfo->spts[0], udpinfo->spts[1],
 			   udpinfo->dpts[0], udpinfo->dpts[1]);
 	}
@@ -1517,6 +1864,7 @@ list_redirect_rule(const char * ifname)
 	const struct ip_nat_multi_range * mr;
 	const char * target_str;
 	char addr[16], mask[16];
+	unsigned int index;
 	(void)ifname;
 
 	h = iptc_init("nat");
@@ -1535,6 +1883,7 @@ list_redirect_rule(const char * ifname)
 #endif
 		return -1;
 	}
+	index = 0;
 #ifdef IPTABLES_143
 	for(e = iptc_first_rule(miniupnpd_nat_chain, h);
 		e;
@@ -1548,10 +1897,10 @@ list_redirect_rule(const char * ifname)
 	{
 		target_str = iptc_get_target(e, &h);
 #endif
-		printf("===\n");
+		printf("=== rule #%u ===\n", index);
 		inet_ntop(AF_INET, &e->ip.src, addr, sizeof(addr));
 		inet_ntop(AF_INET, &e->ip.smsk, mask, sizeof(mask));
-		printf("src = %s%s/%s\n", (e->ip.invflags & IPT_INV_SRCIP)?"! ":"",
+		printf("src = %s%s/%s\t", (e->ip.invflags & IPT_INV_SRCIP)?"! ":"",
 		       /*inet_ntoa(e->ip.src), inet_ntoa(e->ip.smsk)*/
 		       addr, mask);
 		inet_ntop(AF_INET, &e->ip.dst, addr, sizeof(addr));
@@ -1563,7 +1912,7 @@ list_redirect_rule(const char * ifname)
 		printf("in_if = ");
 		print_iface(e->ip.iniface, e->ip.iniface_mask,
 		            e->ip.invflags & IPT_INV_VIA_IN);
-		printf(" out_if = ");
+		printf("\tout_if = ");
 		print_iface(e->ip.outiface, e->ip.outiface_mask,
 		            e->ip.invflags & IPT_INV_VIA_OUT);
 		printf("\n");
@@ -1575,16 +1924,17 @@ list_redirect_rule(const char * ifname)
 			IPT_MATCH_ITERATE(e, print_match);
 			/*printf("\n");*/
 		}
-		printf("target = %s\n", target_str);
+		printf("target = %s :\n", target_str);
 		target = (void *)e + e->target_offset;
 		mr = (const struct ip_nat_multi_range *)&target->data[0];
-		printf("ips ");
+		printf("  ips ");
 		printip(ntohl(mr->range[0].min_ip));
 		printf(" ");
 		printip(ntohl(mr->range[0].max_ip));
-		printf("\nports %hu %hu\n", ntohs(mr->range[0].min.all),
+		printf("\n  ports %hu %hu\n", ntohs(mr->range[0].min.all),
 		          ntohs(mr->range[0].max.all));
-		printf("flags = %x\n", mr->range[0].flags);
+		printf("  flags = %x\n", mr->range[0].flags);
+		index++;
 	}
 	if(h)
 #ifdef IPTABLES_143
@@ -1592,6 +1942,7 @@ list_redirect_rule(const char * ifname)
 #else
 		iptc_free(&h);
 #endif
+	printf("======\n");
 	return 0;
 }
 #endif
