@@ -1,7 +1,7 @@
-/* $Id: iptpinhole.c,v 1.17 2017/11/02 15:41:46 nanard Exp $ */
+/* $Id: iptpinhole.c,v 1.19 2018/04/22 19:36:58 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2012-2017 Thomas Bernard
+ * (c) 2012-2018 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -16,6 +16,7 @@
 #include "../macros.h"
 #include "iptpinhole.h"
 #include "../upnpglobalvars.h"
+#include "../upnputils.h"
 
 #ifdef ENABLE_UPNPPINHOLE
 
@@ -458,7 +459,7 @@ clean_pinhole_list(unsigned int * next_timestamp)
 	time_t current_time;
 	int n = 0;
 
-	current_time = time(NULL);
+	current_time = upnp_time();
 	p = pinhole_list.lh_first;
 	while(p != NULL) {
 		if(p->timestamp <= (unsigned int)current_time) {
