@@ -1,8 +1,8 @@
-/* $Id: upnpredirect.c,v 1.91 2016/02/16 12:15:02 nanard Exp $ */
+/* $Id: upnpredirect.c,v 1.94 2018/05/03 08:27:40 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
- * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2016 Thomas Bernard
+ * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
+ * (c) 2006-2018 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -25,6 +25,7 @@
 #include "upnpglobalvars.h"
 #include "upnpevents.h"
 #include "portinuse.h"
+#include "upnputils.h"
 #if defined(USE_NETFILTER)
 #include "netfilter/iptcrdr.h"
 #endif
@@ -108,6 +109,15 @@ lease_file_add(unsigned short eport,
 		return -1;
 	}
 
+	/* convert our time to unix time
+     * if LEASEFILE_USE_REMAINING_TIME is defined, only the remaining time is stored */
+	if (timestamp != 0) {
+		timestamp -= upnp_time();
+#ifndef LEASEFILE_USE_REMAINING_TIME
+		timestamp += time(NULL);
+#endif
+	}
+
 	fprintf(fd, "%s:%hu:%s:%hu:%u:%s\n",
 	        proto_itoa(proto), eport, iaddr, iport,
 	        timestamp, desc);
@@ -134,8 +144,7 @@ lease_file_remove(unsigned short eport, int proto)
 		return -1;
 	}
 
-	strncpy( tmpfilename, lease_file, sizeof(tmpfilename) );
-	strncat( tmpfilename, "XXXXXX", sizeof(tmpfilename) - strlen(tmpfilename));
+	snprintf( tmpfilename, sizeof(tmpfilename), "%sXXXXXX", lease_file);
 
 	fd = fopen( lease_file, "r");
 	if (fd==NULL) {
@@ -189,6 +198,9 @@ int reload_from_lease_file()
 	unsigned int leaseduration;
 	unsigned int timestamp;
 	time_t current_time;
+#ifndef LEASEFILE_USE_REMAINING_TIME
+	time_t current_unix_time;
+#endif
 	char line[128];
 	int r;
 
@@ -202,7 +214,10 @@ int reload_from_lease_file()
 		syslog(LOG_WARNING, "could not unlink file %s : %m", lease_file);
 	}
 
-	current_time = time(NULL);
+	current_time = upnp_time();
+#ifndef LEASEFILE_USE_REMAINING_TIME
+	current_unix_time = time(NULL);
+#endif
 	while(fgets(line, sizeof(line), fd)) {
 		syslog(LOG_DEBUG, "parsing lease file line '%s'", line);
 		proto = line;
@@ -250,12 +265,18 @@ int reload_from_lease_file()
 			*(p--) = '\0';
 
 		if(timestamp > 0) {
-			if(timestamp <= (unsigned int)current_time) {
+#ifdef LEASEFILE_USE_REMAINING_TIME
+			leaseduration = timestamp;
+			timestamp += current_time;	/* convert to our time */
+#else
+			if(timestamp <= (unsigned int)current_unix_time) {
 				syslog(LOG_NOTICE, "already expired lease in lease file");
 				continue;
 			} else {
-				leaseduration = timestamp - current_time;
+				leaseduration = timestamp - current_unix_time;
+				timestamp = leaseduration + current_time; /* convert to our time */
 			}
+#endif
 		} else {
 			leaseduration = 0;	/* default value */
 		}
@@ -274,10 +295,35 @@ int reload_from_lease_file()
 
 	return 0;
 }
+
+#ifdef LEASEFILE_USE_REMAINING_TIME
+void lease_file_rewrite(void)
+{
+	int index;
+	unsigned short eport, iport;
+	int proto;
+	char iaddr[32];
+	char desc[64];
+	char rhost[40];
+	unsigned int timestamp;
+
+	if (lease_file == NULL) return;
+	remove(lease_file);
+	for(index = 0; ; index++) {
+		if(get_redirect_rule_by_index(index, 0/*ifname*/, &eport, iaddr, sizeof(iaddr),
+		                              &iport, &proto, desc, sizeof(desc),
+		                              rhost, sizeof(rhost), &timestamp,
+		                              0, 0) < 0)
+			break;
+		if(lease_file_add(eport, iaddr, iport, proto, desc, timestamp) < 0)
+			break;
+	}
+}
+#endif
 #endif
 
 /* upnp_redirect()
- * calls OS/fw dependant implementation of the redirection.
+ * calls OS/fw dependent implementation of the redirection.
  * protocol should be the string "TCP" or "UDP"
  * returns: 0 on success
  *          -1 failed to redirect
@@ -353,7 +399,7 @@ upnp_redirect(const char * rhost, unsigned short eport,
 		    (rhost && (strcmp(rhost, rhost_old) == 0)))) {
 			syslog(LOG_INFO, "updating existing port mapping %hu %s (rhost '%s') => %s:%hu",
 				eport, protocol, rhost_old, iaddr_old, iport_old);
-			timestamp = (leaseduration > 0) ? time(NULL) + leaseduration : 0;
+			timestamp = (leaseduration > 0) ? upnp_time() + leaseduration : 0;
 			if(iport != iport_old) {
 				r = update_portmapping(ext_if_name, eport, proto, iport, desc, timestamp);
 			} else {
@@ -378,7 +424,7 @@ upnp_redirect(const char * rhost, unsigned short eport,
 		return -4;
 #endif /* CHECK_PORTINUSE */
 	} else {
-		timestamp = (leaseduration > 0) ? time(NULL) + leaseduration : 0;
+		timestamp = (leaseduration > 0) ? upnp_time() + leaseduration : 0;
 		syslog(LOG_INFO, "redirecting port %hu to %s:%hu protocol %s for: %s",
 			eport, iaddr, iport, protocol, desc);
 		return upnp_redirect_internal(rhost, eport, iaddr, iport, proto,
@@ -425,7 +471,7 @@ upnp_redirect_internal(const char * rhost, unsigned short eport,
 
 
 
-/* Firewall independant code which call the FW dependant code. */
+/* Firewall independent code which call the FW dependent code. */
 int
 upnp_get_redirection_infos(unsigned short eport, const char * protocol,
                            unsigned short * iport,
@@ -448,7 +494,7 @@ upnp_get_redirection_infos(unsigned short eport, const char * protocol,
 	                      0, 0);
 	if(r == 0 &&
 	   timestamp > 0 &&
-	   timestamp > (unsigned int)(current_time = time(NULL))) {
+	   timestamp > (unsigned int)(current_time = upnp_time())) {
 		*leaseduration = timestamp - current_time;
 	} else {
 		*leaseduration = 0;
@@ -481,7 +527,7 @@ upnp_get_redirection_infos_by_index(int index,
 		return -1;
 	else
 	{
-		current_time = time(NULL);
+		current_time = upnp_time();
 		*leaseduration = (timestamp > (unsigned int)current_time)
 		                 ? (timestamp - current_time)
 		                 : 0;
@@ -567,7 +613,7 @@ get_upnp_rules_state_list(int max_rules_number_target)
 	tmp = malloc(sizeof(struct rule_state));
 	if(!tmp)
 		return 0;
-	current_time = time(NULL);
+	current_time = upnp_time();
 	nextruletoclean_timestamp = 0;
 	while(get_redirect_rule_by_index(i, /*ifname*/0, &tmp->eport, 0, 0,
 	                              &iport, &proto, 0, 0, 0,0, &timestamp,
